clear all

% Path to Gammatone filter bank toolbox
addpath('GammatoneBank')

numData = input('Which RawData ? ');
numFilter = input('How many Gammatone filter (16 is good) ? ');
if isempty(numFilter)
    numFilter=16;
end

% Path to RawData
RAWDATA_PATH = ['../NAL_dataset/NAL_dataset' int2str(numData) '/RawData' int2str(numData)];

A = dir([RAWDATA_PATH]);
% Initialization (Change accordingly to
WN_data = zeros(6*numFilter+4,2000000);
Sp_data = zeros(6*numFilter+4,2000000);
WNcol = 0;
Spcol = 0;


% Loop on body orientation
for i = 3:size(A,1)
    
    dataDir = [RAWDATA_PATH  '/' A(i).name];
    docNode = xmlread([dataDir '/SoundRecord.xml']);
    records = docNode.getElementsByTagName('Record');
    
    % Loop on head orientation
    for k = 0:records.getLength-1
        headAngle = records.item(k);
        
        % Adress of the recording
        nodeAdress = headAngle.getElementsByTagName('nameRF').item(0);
        adress = char(nodeAdress.getFirstChild.getData);
        
        % Reading of the sound
        [sound,afs] = audioread([dataDir adress(17:end)]);
        
        % White noise / speech separation
        last = size(sound,1);
        if numData  ==  1
            WN_sound = sound(last-3.5*afs:last-2.5*afs,:);
            Sp_sound = sound(15000:last-4*afs,:);
        else
            WN_sound = sound(last-2.5*afs:last-1.5*afs,:);
            Sp_sound = sound(7000:last-3.5*afs,:);
        end
        
        % Loop on white noise / speech
        for soundType = 0:1
            
            if soundType == 0 % White noise
                sound = WN_sound;
            elseif soundType == 1 % Speech
                sound = Sp_sound;
            end
            
            % Filtering recordings from the 2 front microphones only
            % Gammatone filters from 300Hz to 8000Hz
            [y3,~,~] = gammatonebank(sound(:,3),300,8000,numFilter,afs);
            [y4,~,~] = gammatonebank(sound(:,4),300,8000,numFilter,afs);
            y3 = y3';
            y4 = y4';
            
            % Cutting the signal (windowing)
            T = size(y3,1); % length of the sound in samples
            tWin = 30e-3; % length of the window in second
            nWin = ceil(tWin*afs); % length of the window in samples
            cover = 0.5; % Covering rate
            nbWin = ceil(((T-nWin)/((1-cover)*nWin))+1);
            
            % Loop on the windows
            for j = 1:nbWin
                
                % Extraction of the window
                shift = round((1-cover)*nWin); % Window shift at each iteration
                if (j-1)*shift+1+nWin>T
                    y3Win = y3((j-1)*shift+1:end,:);
                    y4Win = y4((j-1)*shift+1:end,:);
                else
                    y3Win = y3((j-1)*shift+1:(j-1)*shift+1+nWin,:);
                    y4Win = y4((j-1)*shift+1:(j-1)*shift+1+nWin,:);
                end
                
                % Energy of the sound
                energy3 = sum(y3Win.^2)';
                energy4 = sum(y4Win.^2)';
                
                % Delays of the sound
                for l = 1:size(energy3,1)
                    
                    % Crosscorrelation
                    c = xcorr(y3Win(:,l),y4Win(:,l),'coeff');
                    middle = size(y3Win,1) + 1;
                    [~,Tau] = max(abs(c));
                    [picAbs,picOrd] = parabInterpol(abs(c(Tau-1)),abs(c(Tau)),abs(c(Tau+1))); % Better delay approximation
                    delay(l,1) = (Tau+picAbs-middle)/afs;
                    trust(l,1) = picOrd;
                    
                    [~,Tau16] = max(abs(c(middle-16:middle+16))); % Maximum without reverberation
                    Tau16 = Tau16+middle-16;
                    [picAbs16,picOrd16] = parabInterpol(abs(c(Tau16-1)),abs(c(Tau16)),abs(c(Tau16+1)));
                    delay16(l,1) = (Tau16+picAbs16-middle)/afs;
                    trust16(l,1) = picOrd16;
                    
                end
                
                % Source azimuth angle calculation
                if numData == 1 || numData == 2
                    bodyAngle = str2num(A(i).name(17:19));
                    sourceHeight = 81;
                    sourceDistance = 144;
                elseif numData == 3
                    fixedAngle = str2num(A(i).name(18:20));
                    mobileAngle = str2num(A(i).name(23:25));
                    sourceHeight = str2num(A(i).name(28:30));
                    sourceDistance = str2num(A(i).name(33:35));
                    bodyAngle = mobileAngle-fixedAngle;
                else
                    bodyAngle = str2num(A(i).name(22:24));
                    sourceHeight = 81;
                    naoPosition=A(i).name(17:20);
                    if strcmp(naoPosition,'+0+0')
                        sourceDistance = 140;
                    elseif strcmp(naoPosition,'+0-2')
                        sourceDistance = 171;
                    elseif strcmp(naoPosition,'+0+2')
                        sourceDistance = 185;
                    elseif strcmp(naoPosition,'+1+1')
                        sourceDistance = 114;
                    elseif strcmp(naoPosition,'+1-1')
                        sourceDistance = 104;
                    elseif strcmp(naoPosition,'-1+1')
                        sourceDistance = 213;
                    elseif strcmp(naoPosition,'-1-1')
                        sourceDistance = 209;
                    end
                end
                headYawNode = headAngle.getElementsByTagName('realHeadYawPosition').item(0);
                headYaw = str2num(char(headYawNode.getFirstChild.getData));
                sourceAzimuth = bodyAngle-(headYaw*180/pi);
                
                % Source elevation angle calculation
                headPitchNode = headAngle.getElementsByTagName('realHeadPitchPosition').item(0);
                headPitch = str2num(char(headPitchNode.getFirstChild.getData));
                sourceElevation =(atan((sourceHeight-45)/sourceDistance)-headPitch)*180/pi;
                
                % Data matrix
                if soundType == 0
                    WNcol = WNcol + 1;
                    WN_data(1:6*numFilter+4,WNcol) = [energy3 ; energy4 ; delay ; delay16 ; trust ; trust16 ; headYaw ; headPitch ; sourceAzimuth ; sourceElevation];
                    if not(mod(WNcol,100))
                        fprintf(['White noise matrix filled up to column ' int2str(WNcol) '\n' ])
                    end
                elseif soundType == 1
                    Spcol = Spcol + 1;
                    Sp_data(1:6*numFilter+4,Spcol) = [energy3 ; energy4 ; delay ; delay16 ; trust ; trust16 ; headYaw ; headPitch ; sourceAzimuth ; sourceElevation];
                end
                
            end
            
        end
        
    end
    
end

WN_data = WN_data(:,1:WNcol);
Sp_data = Sp_data(:,1:Spcol);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
save(['./Data/Data_' int2str(numFilter) 'Gammatone_Dataset' int2str(numData) ],'WN_data','Sp_data')
